@@ -100,6 +100,10 @@ CLOUD_BUCKET_CACHE = MODEL_CACHE_PATH
 HF_SOURCE_DIR = Path("./root/.cache/huggingface/hub/")
 HF_CACHE_DIR = Path("/root/.cache/huggingface/hub/")
 
+# PORT: knative reserved
+# HOSTNAME: set to the pod name by k8s
+K8S_RESERVED_ENVIRONMENT_VARIABLES = set(["PORT", "HOSTNAME"])
+
 
 class RemoteCache(ABC):
     def __init__(self, repo_name, data_dir, revision=None):
@@ -339,12 +343,11 @@ def build_model_cache_v2_and_copy_bptr_manifest(config: TrussConfig, build_dir:
         ]
         # create BasetenPointer from models
         basetenpointer_json = create_basetenpointer_from_models(models=py_models)
-        pointers = json.loads(basetenpointer_json)
-        basetenpointer_list = json.dumps({"pointers": pointers}, indent=2)
-        logging.info(f"debug: created {len(pointers)} number of BasetenPointers")
-        logging.info(f"pointers json: {basetenpointer_list}")
+        bptr_py = json.loads(basetenpointer_json)["pointers"]
+        logging.info(f"created ({len(bptr_py)}) Basetenpointer")
+        logging.info(f"pointers json: {basetenpointer_json}")
         with open(build_dir / "bptr-manifest", "w") as f:
-            f.write(basetenpointer_list)
+            f.write(basetenpointer_json)
     except Exception as e:
         logging.warning(f"debug: failed to create BasetenPointer: {e}")
         # TODO: remove below section + remove logging lines above.
@@ -622,12 +625,6 @@ class ServingImageBuilder(ImageBuilder):
             )
 
         if config.model_cache.is_v2:
-            if config.trt_llm:
-                raise RuntimeError(
-                    "TensorRTLLM models is already occupying and using `model_cache` by default. "
-                    "Additional huggingface weights are not allowed. "
-                    "Feel free to reach out to us if you need this feature."
-                )
             logging.info(
                 f"`model_cache` with `use_volume=True` is enabled. Creating {config.model_cache}"
             )
@@ -728,16 +725,28 @@ class ServingImageBuilder(ImageBuilder):
         if build_hash_path.exists():
             shutil.rmtree(build_hash_path)
         shutil.copytree(build_dir, build_hash_path)
-        # remove the config.yaml file from the build_hash directory
-        # we will use only config_build_time.yaml to compute the hash
+
+        # Clear runtime attributes, which will produce a sanitized copy of the original TrussConfig,
+        # used to determine if we need to rebuild the image or not.
         config_file_path = build_hash_path / "config.yaml"
         if config_file_path.exists():
-            config_file_path.unlink()
-        # similarly, remove the build-time config from the context directory
-        # else it will clobber prior hashes
-        config_build_time_file_path = build_dir / "config_build_time.yaml"
-        if config_build_time_file_path.exists():
-            config_build_time_file_path.unlink()
+            truss_config = TrussConfig.from_yaml(config_file_path)
+            truss_config.clear_runtime_fields()
+            truss_config.write_to_yaml_file(config_file_path)
+
+    def _filter_reserved_environment_variables(
+        self, config: TrussConfig
+    ) -> Dict[str, str]:
+        # Some environment variables are reserved by the k8s runtime.
+        # Pass them through to be baked into the container image.
+        passthrough_environment_variables = {}
+        if config.environment_variables:
+            passthrough_environment_variables = {
+                k: v
+                for k, v in config.environment_variables.items()
+                if k in K8S_RESERVED_ENVIRONMENT_VARIABLES
+            }
+        return passthrough_environment_variables
 
     def _render_dockerfile(
         self,
@@ -780,6 +789,10 @@ class ServingImageBuilder(ImageBuilder):
         max_py_version = packaging.version.parse(SUPPORTED_PYTHON_VERSIONS[-1])
 
         hf_access_token = config.secrets.get(constants.HF_ACCESS_TOKEN_KEY)
+        passthrough_environment_variables = self._filter_reserved_environment_variables(
+            config
+        )
+
         dockerfile_contents = dockerfile_template.render(
             should_install_server_requirements=should_install_server_requirements,
             base_image_name_and_tag=base_image_name_and_tag,
@@ -812,6 +825,7 @@ class ServingImageBuilder(ImageBuilder):
             external_data_files=external_data_files,
             build_commands=build_commands,
             use_local_src=config.use_local_src,
+            passthrough_environment_variables=passthrough_environment_variables,
             **FILENAME_CONSTANTS_MAP,
         )
         # Consolidate repeated empty lines to single empty lines.
